"""Collection of utils."""
import pathlib
from typing import Any, Dict, List, Tuple, Union

import toml
import yaml

from ihsan.schema import ADFHActionsType, ADFHFieldsType, ADFHModelsType


def read_adfh_file(file: str) -> Tuple[Union[Dict, str], bool]:
    """Parse an ADFH file into dict."""
    file_path = pathlib.Path(file)
    file_name, file_extension = file_path.name.rsplit(".")

    if file_path.exists() and file_path.is_file():

        if file_extension in ["yaml", "yml"]:
            with open(file_path, "r") as f:
                data = yaml.safe_load(f)
            return data, False

        elif file_extension == "toml":
            data = toml.load(f"{file_path}")
            return data, False
        else:
            return "You can only pick toml or yaml file.", True
    return "File doesn't exist.", True


def sdl_data_type_converter(data: str) -> str:
    """Converting ADFH data type into sdl data type."""
    types = {
        "unique id": "String",
        "text": "String",
        "checkbox": "Boolean",
        "number": "Int",
    }
    return types.get(data, "String")


def find_action(actions: List[ADFHActionsType], command: str) -> List[Dict[str, Any]]:
    """Search for a certain action."""
    return [action.dict() for action in actions if action.type == command]


def find_field(fields: List[ADFHFieldsType], field_id: str) -> ADFHFieldsType:
    """Search for a certain field."""
    field_dict = {}
    for field in fields:
        if field.id == field_id:
            data_type = sdl_data_type_converter(field.type)
            field_dict.update(
                {
                    "id": field.id,
                    "name": field.name,
                    "type": data_type,
                    "mandatory": "!" if field.mandatory == "yes" else "",
                }
            )
    return ADFHFieldsType(**field_dict)


def find_model(fields: List[ADFHModelsType], model_id: str) -> ADFHModelsType:
    """Search for a certain model."""
    model_dict = {}
    for field in fields:
        if field.id == model_id:
            model_dict.update({"id": field.id, "name": field.name})
    return ADFHModelsType(**model_dict)



"""Collection of schemas."""
from typing import List, Optional

from pydantic import BaseModel, Field


class ADFHExtraType(BaseModel):
    """Schema for ADFH extra."""

    type: str
    name: str
    value: str
    tags: Optional[List[str]]


class ADFHFieldsType(BaseModel):
    """Schema for ADFH fields."""

    id: str
    name: str
    type: str
    mandatory: str = "no"
    options: Optional[List]
    text: Optional[str]


class ADFHModelsPropertiesType(BaseModel):
    """Schema for ADFH models properties."""

    model: str
    assign: str


class ADFHModelsType(BaseModel):
    """Schema for ADFH models."""

    id: str
    name: str
    properties: Optional[List[ADFHModelsPropertiesType]]
    text: Optional[str]


class ADFHActionsInputType(BaseModel):
    """Schema for ADFH action input."""

    action: str
    assign: str


class ADFHActionsType(BaseModel):
    """Schema for ADFH action."""

    id: str
    name: str
    type: str
    model: str
    input: Optional[List[ADFHActionsInputType]]
    subject: Optional[str]
    text: Optional[str]


class ADFHType(BaseModel):
    """Schema for ADFH."""

    version: str
    extra: Optional[List[ADFHExtraType]]
    fields_list: List[ADFHFieldsType] = Field(..., alias="fields")
    models: List[ADFHModelsType]
    actions: Optional[List[ADFHActionsType]]


class IhsanType(BaseModel):
    """Schema for Ihsan."""

    adfh: ADFHType



"""Translator module where it change ADFH into json, SDL, openAPI, etc..."""
from .schema import IhsanType
from .utils import find_action, find_field, find_model


def to_sdl(data: IhsanType, indention: int = 4) -> str:
    """Function that transfer ADFH into SDL aka GraphQL schema."""
    show_me_list = find_action(data.adfh.actions, "show me list")
    let_me_remove = find_action(data.adfh.actions, "let me remove")
    let_me_add = find_action(data.adfh.actions, "let me add")
    text = ""
    for model in data.adfh.models:
        text += f"type {model.name} {'{'}\n"

        for pro in model.properties:
            pro = find_field(data.adfh.fields_list, pro.assign)
            placeholder = f"{pro.name}: {pro.type}{pro.mandatory}\n"
            text += placeholder.rjust(len(placeholder) + indention)
        text += "}\n"

    text += "type Query {\n"
    for item in show_me_list:
        model = find_model(data.adfh.models, item.get("model"))
        placeholder = f"{item.get('name')}: [{model.name}]\n"
        text += placeholder.rjust(len(placeholder) + indention)
    text += "}\n"

    text += "type Mutation {\n"
    for item in let_me_add:
        placeholder = f"{item.get('name')}("
        text += placeholder.rjust(len(placeholder) + indention)
        for input_action in item.get("input"):
            input_action = find_field(data.adfh.fields_list, input_action.get("assign"))
            text += (
                f"{input_action.name}: {input_action.type}{input_action.mandatory}, "
            )

        model = find_model(data.adfh.models, item.get("model"))
        text += f"): {model.name}\n"

    for item in let_me_remove:
        placeholder = f"{item.get('name')}("
        text += placeholder.rjust(len(placeholder) + indention)
        field = find_field(data.adfh.fields_list, item.get("subject"))
        text += f"{field.name}: {field.type}{field.mandatory}, "
        model = find_model(data.adfh.models, item.get("model"))
        text += f"): {model.name}\n"

    text += "}\n"

    text += """schema {
    query: Query,
    mutation: Mutation
}
    """
    return text



"""Command-line interface."""
import typer
from rich.console import Console

from ihsan import __version__
from ihsan.schema import IhsanType
from ihsan.translation import to_sdl
from ihsan.utils import read_adfh_file

app = typer.Typer(help="Ihsan CLI.")


@app.command()
def version() -> None:
    """Show project Version."""
    console = Console()
    project_name = "Ihsan"
    console.print(f"{project_name} Version: {__version__}", style="bold green")


@app.command("sdl")
def sdl(
    file: str,
    output: str = typer.Option(None, help="save output into a file."),
    folder: bool = typer.Option(False, help="Will open the out file folder."),
) -> None:
    """Generate SDL aka GraphQL schema from ADFH file.

    Args:
        file: Path to ADFH file.
        output: If --output is used, it will save it in file.
        folder: If --output and --folder is used, it will open the saved file.
    """
    console = Console()
    data, is_error = read_adfh_file(file=file)

    if is_error:
        console.print(data, style="bold red")

    else:
        ihsan_type = IhsanType(**data)
        sdl_output = to_sdl(data=ihsan_type)
        if output:
            typer.confirm(
                f"The output will be saved in {output}, are you sure?", abort=True
            )
            with open(output, "w") as output_file:
                output_file.write(sdl_output)
            if folder:
                typer.launch(output, locate=True)
        else:
            console.print(sdl_output, style="bold green")
        console.print(
            "Use -> https://app.graphqleditor.com/ to test the schema :)",
            style="bold blue",
        )


if __name__ == "__main__":  # pragma: no cover
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ihsan']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.3.1,<6.0.0',
 'pydantic>=1.7.2,<2.0.0',
 'rich>=9.2.0,<10.0.0',
 'toml>=0.10.2,<0.11.0',
 'typer[all]>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['ihsan = ihsan.manage:app']}

setup_kwargs = {
    'name': 'ihsan',
    'version': '0.0.2',
    'description': 'Behold Ihsan Project',
    'long_description': "> ## 🛠 Status: In Development\n> Ihsan is currently in development. So we encourage you to use it and give us your feedback, but there are things that haven't been finalized yet and you can expect some changes.\n>\n> See the list of Known Issues and TODOs, below, for updates.\n\n\n## Overview\n\nBehold Ihsan Project\n\n\n## Installation\n\n```shell script\npoetry add ihsan\n\n```\n\nor\n\n```shell script\npip install ihsan\n\n```\n",
    'author': 'Mohamed Nesredin',
    'author_email': 'm.n.kaizen@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Mohamed-Kaizen/ihsan/',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
